/*
 * Copyright (c) 2010 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <math.h>
#include <stdio.h>

#include <portLib.h>
#include <h2mathLib.h>

#include "rmp400Const.h"
#include "server/rmp400Header.h"

static POSTER_ID attitudePosterId = NULL;

static STATUS readAttitudeEstimation(void);


/* rmp400OdometryMethodExec  -  codel EXEC de OdometryMethod
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rmp400SetOdometryCommandExec(RMP400_ODOMETRY_METHOD *params, int *bilan)
{
	if (params->mode == RMP400_ODO_3D) {
		SDI_F->odometryMode = RMP400_ODO_2D;
		
		if (rmp400POM_SENSOR_POSPosterFind(params->attitudePosterName, 
			&attitudePosterId) == ERROR) {
			*bilan = S_rmp400_CANNOT_READ_POSTER;
			return ETHER;
		}
		
		if (readAttitudeEstimation()) {
			*bilan = S_rmp400_CANNOT_READ_POSTER;
			return ETHER;
		}
		
		SDI_F->odometryMode = RMP400_ODO_3D;
	}
	
	return ETHER;
}


/*
 * Read attitude estimation poster
 */
static STATUS
readAttitudeEstimation(void)
{
	POM_SENSOR_POS full_pos;

	if (rmp400POM_SENSOR_POSPosterRead(attitudePosterId, 
		&full_pos) == ERROR) {
		/* Fall back to ODO 2D */
		printf("rmp400OdometryMethod: Cannot read attitude "
		    "estimation poster\n");
		return ERROR;
	}
	SDI_F->attitude = full_pos.sensorToMain.euler;
	return OK;
}

STATUS
rmp400Odo3d(int *report, double period)
{
	GENPOS_CART_STATE *robot = &SDI_F->robot;
	RMP400_CART_3DSTATE *robot3d = &SDI_F->robot3d;
	RMP400_STATUS_STR *status = &SDI_F->status;
	POM_EULER *attitude = &SDI_F->attitude;
	static double mpitch = 0.0, oldPitch = -1.0;

	/* 3D odometry */
	if (SDI_F->odometryMode == RMP400_ODO_3D) {
		if (readAttitudeEstimation() == ERROR) {
			printf("switching back to odo2D\n");
			SDI_F->odometryMode = RMP400_ODO_2D;
		}
	} else {
		mpitch = 0;
		oldPitch = -1;
		memset(attitude, 0, sizeof(POM_EULER));
	}
	robot3d->pitch = attitude->pitch;
	robot3d->roll = attitude->roll;
	/* XXXX For now keep the position in the Z=0 frame */
	robot3d->theta  = robot->theta;
	robot3d->xRob = robot->xRob;
	robot3d->yRob = robot->yRob;
	robot3d->zRob += - (robot->v * period) * sin(robot3d->pitch);
	robot3d->xRef = robot->xRef;
	robot3d->yRef = robot->yRef;
	robot3d->zRef = 0.0;
	robot3d->v = robot->v;
	robot3d->vt = 0.0;
	robot3d->w = robot->w;

	return OK;
}
